@@ -25,20 +25,25 @@
 //! [signal-hook]: https://docs.rs/signal-hook
 #![deny(missing_docs, unsafe_code, rust_2018_idioms)]
 
+use dashmap::DashMap;
+use once_cell::sync::Lazy;
 use std::{
     io,
+    marker::PhantomData,
     path::{Path, PathBuf},
     sync::atomic::AtomicUsize,
 };
 
-use dashmap::DashMap;
-use once_cell::sync::Lazy;
-use tempfile::NamedTempFile;
-
 mod fs;
 pub use fs::{create_dir, remove_dir};
+
 mod handler;
-mod registration;
+
+mod forksafe;
+pub(crate) use forksafe::ForksafeTempfile;
+
+pub mod registration;
+use crate::registration::{Closed, Writable};
 
 static SIGNAL_HANDLER_MODE: AtomicUsize = AtomicUsize::new(SignalHandlerMode::default() as usize);
 static NEXT_MAP_INDEX: AtomicUsize = AtomicUsize::new(0);
@@ -133,42 +138,36 @@ impl AutoRemove {
 /// state forever.
 ///
 /// This kind of raciness exists whenever [`take()`][Registration::take()] is used and can't be circumvented.
-pub struct Registration {
+pub struct Registration<Marker> {
     id: usize,
+    _marker: PhantomData<Marker>,
 }
 
-struct ForksafeTempfile {
-    inner: NamedTempFile,
-    cleanup: AutoRemove,
-    owning_process_id: u32,
-}
-
-impl ForksafeTempfile {
-    fn new(inner: NamedTempFile, cleanup: AutoRemove) -> Self {
-        ForksafeTempfile {
-            inner,
-            cleanup,
-            owning_process_id: std::process::id(),
-        }
-    }
-}
-
-/// A shortcut to [`Registration::new()`].
+/// A shortcut to [`Registration::<Writable>::new()`], creating a writable temporary file with non-clashing name in a directory.
 pub fn new(
     containing_directory: impl AsRef<Path>,
     directory: ContainingDirectory,
     cleanup: AutoRemove,
-) -> io::Result<Registration> {
-    Registration::new(containing_directory, directory, cleanup)
+) -> io::Result<Registration<Writable>> {
+    Registration::<Writable>::new(containing_directory, directory, cleanup)
 }
 
-/// A shortcut to [`Registration::at_path()`].
-pub fn at_path(
+/// A shortcut to [`Registration::<Writable>::at_path()`] providing a writable temporary file at the given path.
+pub fn writable_at(
     path: impl AsRef<Path>,
     directory: ContainingDirectory,
     cleanup: AutoRemove,
-) -> io::Result<Registration> {
-    Registration::at_path(path, directory, cleanup)
+) -> io::Result<Registration<Writable>> {
+    Registration::<Writable>::at(path, directory, cleanup)
+}
+
+/// A shortcut to [`Registration::<Closed>::at_path()`] providing a closed temporary file to mark the presence of something.
+pub fn mark_at(
+    path: impl AsRef<Path>,
+    directory: ContainingDirectory,
+    cleanup: AutoRemove,
+) -> io::Result<Registration<Closed>> {
+    Registration::<Closed>::at(path, directory, cleanup)
 }
 
 /// Explicitly (instead of lazily) initialize signal handlers and other state to keep track of tempfiles.
